@@ -34,19 +34,22 @@ module ActiveRecord
         end
 
         def klass(value)
-          case value
-          when Base
+          if value.is_a?(Base)
             value.class
-          when Relation
-            value.klass
+          elsif value.is_a?(Relation)
+            value.model
           end
         end
 
         def convert_to_id(value)
-          case value
-          when Base
-            value._read_attribute(primary_key(value))
-          when Relation
+          if value.is_a?(Base)
+            primary_key = primary_key(value)
+            if primary_key.is_a?(Array)
+              primary_key.map { |column| value._read_attribute(column) }
+            else
+              value._read_attribute(primary_key)
+            end
+          elsif value.is_a?(Relation)
             value.select(primary_key(value))
           else
             value
